@@ -29,13 +29,14 @@ class Control: public HardwareManipulator
 Q_OBJECT
 
 public:
-    Control(const QString& ruleFile, QObject *parent=0);
+    Control(const QString& ruleFile, SimRules *sr, QObject *parent=0);
     virtual ~Control();
 
 public slots:
     void handleFromData( const QString& );
     void handleToData( const QString& );
     void setPhoneNumber( const QString& );
+    void handleNewApp();
 
 protected:
     virtual void warning( const QString&, const QString& );
@@ -48,8 +49,8 @@ private:
 class ControlFactory : public HardwareManipulatorFactory
 {
 public:
-    inline virtual HardwareManipulator *create(QObject *parent)
-        { return new Control(ruleFile(), parent); }
+    inline virtual HardwareManipulator *create(SimRules *sr, QObject *parent)
+        { return new Control(ruleFile(), sr, parent); }
 };
 
 
