@@ -26,7 +26,6 @@ class SimApplicationPrivate
 public:
     SimApplicationPrivate()
     {
-        rules = 0;
         expectedType = QSimCommand::NoCommand;
         target = 0;
         slot = 0;
@@ -41,10 +40,11 @@ public:
     bool inResponse;
 };
 
-SimApplication::SimApplication( QObject *parent )
+SimApplication::SimApplication( SimRules *rules, QObject *parent )
     : QObject( parent )
 {
     d = new SimApplicationPrivate();
+    d->rules = rules;
 }
 
 SimApplication::~SimApplication()
@@ -80,8 +80,7 @@ void SimApplication::command( const QSimCommand& cmd,
     // the middle of processing a TERMINAL RESPONSE or ENVELOPE,
     // then delay the unsolicited notification until later.
     if ( d->rules && !d->inResponse ) {
-        d->rules->unsolicited
-            ( "*TCMD: " + QString::number( d->currentCommand.size() ) );
+        d->rules->proactiveCommandNotify( d->currentCommand );
     }
 }
 
@@ -90,11 +89,8 @@ void SimApplication::command( const QSimCommand& cmd,
 */
 void SimApplication::controlEvent( const QSimControlEvent& event )
 {
-    if ( d->rules ) {
-        d->rules->unsolicited
-            ( "*TCC: " + QString::number( (int)(event.type()) ) +
-              "," + QAtUtils::toHex( event.toPdu() ) );
-    }
+    if ( d->rules )
+        d->rules->callControlEventNotify( event );
 }
 
 /*!
@@ -123,7 +119,7 @@ void SimApplication::abort()
     d->currentCommand = QByteArray();
     d->target = 0;
     d->slot = 0;
-    mainMenu();
+    endSession();
 }
 
 /*!
@@ -144,7 +140,7 @@ void SimApplication::abort()
 void SimApplication::mainMenuSelection( int id )
 {
     Q_UNUSED(id);
-    mainMenu();
+    endSession();
 }
 
 /*!
@@ -157,102 +153,52 @@ void SimApplication::mainMenuSelection( int id )
 void SimApplication::mainMenuHelpRequest( int id )
 {
     Q_UNUSED(id);
-    mainMenu();
+    endSession();
 }
 
-void SimApplication::setSimRules( SimRules *rules )
+bool SimApplication::envelope( const QSimEnvelope& env )
 {
-    d->rules = rules;
-}
-
-bool SimApplication::execute( const QString& cmd )
-{
-    // Process SIM toolkit begin and end commands by forcing the app back to the main menu.
-    if ( cmd == "AT*TSTB" || cmd == "AT*TSTE") {
-        d->rules->respond( "OK" );
-        abort();
+    /* Process a menu selection ENVELOPE message.  We turn it into a
+     * QSimTerminalResponse to make it easier to process.  */
+    if ( env.type() == QSimEnvelope::EventDownload )
         return true;
-    }
 
-    // If not AT+CSIM, then this is not a SIM toolkit command.
-    if ( !cmd.startsWith( "AT+CSIM=" ) )
+    if ( env.type() != QSimEnvelope::MenuSelection )
+        /* Not supported */
         return false;
 
-    // Extract the binary payload of the AT+CSIM command.
-    int comma = cmd.indexOf( QChar(',') );
-    if ( comma < 0 )
-        return false;
-    QByteArray param = QAtUtils::fromHex( cmd.mid(comma + 1) );
-    if ( param.length() < 5 || param[0] != (char)0xA0 )
+    if ( d->expectedType != QSimCommand::SetupMenu )
+        /* Envelope sent for the wrong type of command. */
         return false;
 
-    // Check for TERMINAL PROFILE, FETCH, TERMINAL RESPONSE,
-    // and ENVELOPE packets.
-    if ( param[1] == (char)0x10 ) {
-        // Download of a TERMINAL PROFILE.  We respond with a simple OK,
-        // on the assumption that what we were sent was valid.
-        d->rules->respond( "+CSIM: 4,9000\\n\\nOK" );
-
-        // Abort the SIM application and force it to return to the main menu.
-        abort();
-    } else if ( param[1] == (char)0x12 ) {
-        // Fetch the current command contents.
-        QByteArray resp = d->currentCommand;
-        if ( resp.isEmpty() ) {
-            // We weren't expecting a FETCH.
-            d->rules->respond( "+CSIM: 4,6F00\\n\\nOK" );
-            return true;
-        }
-        resp += (char)0x90;
-        resp += (char)0x00;
-        d->rules->respond( "+CSIM: " + QString::number( resp.size() * 2 ) + "," +
-                           QAtUtils::toHex( resp ) + "\\n\\nOK" );
-    } else if ( param[1] == (char)0x14 ) {
-        // Process a TERMINAL RESPONSE message.
-        QSimTerminalResponse resp;
-        resp = QSimTerminalResponse::fromPdu( param.mid(5) );
-        if ( resp.command().type() != QSimCommand::NoCommand &&
-             resp.command().type() != d->expectedType ) {
-            // Response to the wrong type of command.
-            d->rules->respond( "+CSIM: 4,6F00\\n\\nOK" );
-            return true;
-        }
-        response( resp );
-    } else if ( param[1] == (char)0xC2 ) {
-        // Process a menu selection ENVELOPE message.  We turn it into a
-        // QSimTerminalResponse to make it easier to process.
-        QSimEnvelope env;
-        env = QSimEnvelope::fromPdu( param.mid(5) );
-        if ( env.type() == QSimEnvelope::EventDownload ) {
-            d->rules->respond( "+CSIM: 4,9000\\n\\nOK" );
-            return true;
-        }
-        if ( env.type() != QSimEnvelope::MenuSelection )
-            return false;
-        if ( d->expectedType != QSimCommand::SetupMenu ) {
-            // Envelope sent for the wrong type of command.
-            d->rules->respond( "+CSIM: 4,6F00\\n\\nOK" );
-            return true;
-        }
-        d->rules->respond( "+CSIM: 4,9000\\n\\nOK" );
-        d->expectedType = QSimCommand::NoCommand;
-        d->currentCommand = QByteArray();
-        d->target = 0;
-        d->slot = 0;
-        if ( env.requestHelp() )
-            mainMenuHelpRequest( env.menuItem() );
-        else
-            mainMenuSelection( env.menuItem() );
-    } else {
-        // This SIM command is not related to SIM toolkit - ignore it.
-        return false;
-    }
+    d->expectedType = QSimCommand::NoCommand;
+    d->currentCommand = QByteArray();
+    d->target = 0;
+    d->slot = 0;
+    if ( env.requestHelp() )
+        mainMenuHelpRequest( env.menuItem() );
+    else
+        mainMenuSelection( env.menuItem() );
 
     return true;
 }
 
-void SimApplication::response( const QSimTerminalResponse& resp )
+QByteArray SimApplication::fetch( bool clear )
 {
+    QByteArray resp = d->currentCommand;
+
+    if ( clear )
+        d->currentCommand = QByteArray();
+
+    return resp;
+}
+
+bool SimApplication::response( const QSimTerminalResponse& resp )
+{
+    if ( resp.command().type() != QSimCommand::NoCommand &&
+         resp.command().type() != d->expectedType )
+        return false;
+
     // Save the target information.
     QObject *target = d->target;
     const char *slot = d->slot;
@@ -284,7 +230,7 @@ void SimApplication::response( const QSimTerminalResponse& resp )
 
     // Answer the AT+CSIM command and send notification of the new command.
     if ( !d->rules )
-        return;
+        return false;////
     if ( d->currentCommand.isEmpty() || resp.command().type() == QSimCommand::SetupMenu ) {
         // No new command, so respond with a simple OK.
         d->rules->respond( "+CSIM: 4,9000\\n\\nOK" );
@@ -300,10 +246,18 @@ void SimApplication::response( const QSimTerminalResponse& resp )
         d->rules->unsolicited
             ( "*TCMD: " + QString::number( d->currentCommand.size() ) );
     }
+
+    return true;
 }
 
-DemoSimApplication::DemoSimApplication( QObject *parent )
-    : SimApplication( parent )
+void SimApplication::endSession()
+{
+    d->expectedType = QSimCommand::SetupMenu;
+    d->rules->unsolicited("*TEND");
+}
+
+DemoSimApplication::DemoSimApplication( SimRules *rules, QObject *parent )
+    : SimApplication( rules, parent )
 {
 }
 
@@ -311,9 +265,14 @@ DemoSimApplication::~DemoSimApplication()
 {
 }
 
+const QString DemoSimApplication::getName()
+{
+    return "Demo SIM Application";
+}
+
 #define MainMenu_News       1
 #define MainMenu_Sports     2
-#define MainMenu_Time       3
+#define MainMenu_Calls      3
 #define MainMenu_SticksGame 4
 #define MainMenu_Tones      5
 #define MainMenu_Icons      6
@@ -326,6 +285,10 @@ DemoSimApplication::~DemoSimApplication()
 #define SportsMenu_Snakes       3
 #define SportsMenu_Main         4
 
+#define CallsMenu_Normal        1
+#define CallsMenu_Disconnect    2
+#define CallsMenu_Hold          3
+
 void DemoSimApplication::mainMenu()
 {
     QSimCommand cmd;
@@ -342,8 +305,8 @@ void DemoSimApplication::mainMenu()
     item.setLabel( "Sports" );
     items += item;
 
-    item.setIdentifier( MainMenu_Time );
-    item.setLabel( "Time" );
+    item.setIdentifier( MainMenu_Calls );
+    item.setLabel( "Calls" );
     items += item;
 
     item.setIdentifier( MainMenu_SticksGame );
@@ -411,12 +374,9 @@ void DemoSimApplication::mainMenuSelection( int id )
         }
         break;
 
-        case MainMenu_Time:
+        case MainMenu_Calls:
         {
-            cmd.setType( QSimCommand::SetupCall );
-            cmd.setNumber( "1194" );
-            cmd.setText( "Dialing the Time Guy ..." );
-            command( cmd, this, SLOT(mainMenu()) );
+            sendCallsMenu();
         }
         break;
 
@@ -464,7 +424,7 @@ void DemoSimApplication::mainMenuSelection( int id )
         default:
         {
             // Don't know what this item is, so just re-display the main menu.
-            mainMenu();
+            endSession();
         }
         break;
     }
@@ -536,14 +496,92 @@ void DemoSimApplication::sportsMenu( const QSimTerminalResponse& resp )
             }
             break;
 
-            default:    mainMenu(); break;
+            default:
+                endSession();
+                break;
         }
-    } else if ( resp.result() == QSimTerminalResponse::BackwardMove ) {
-        // Request to move backward.
-        mainMenu();
     } else {
         // Unknown response - just go back to the main menu.
-        mainMenu();
+        endSession();
+    }
+}
+
+void DemoSimApplication::sendCallsMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "Setup Call Menu" );
+
+    item.setIdentifier( CallsMenu_Normal );
+    item.setLabel( "Normal" );
+    items += item;
+
+    item.setIdentifier( CallsMenu_Disconnect );
+    item.setLabel( "Disconnect other calls first" );
+    items += item;
+
+    item.setIdentifier( CallsMenu_Hold );
+    item.setLabel( "Hold other calls first" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(callsMenu(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::callsMenu( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() == QSimTerminalResponse::Success ) {
+        // Item selected.
+        switch ( resp.menuItem() ) {
+
+            case CallsMenu_Normal:
+            {
+                cmd.setType( QSimCommand::SetupCall );
+                cmd.setDestinationDevice( QSimCommand::Network );
+                cmd.setNumber( "1194" );
+                cmd.setText( "Call 1194?" );
+		cmd.setOtherText( "Normal call" );
+                command( cmd, this, SLOT(endSession()) );
+            }
+            break;
+
+            case CallsMenu_Disconnect:
+            {
+                cmd.setType( QSimCommand::SetupCall );
+                cmd.setDestinationDevice( QSimCommand::Network );
+                cmd.setNumber( "1194" );
+                cmd.setText( "Call 1194?" );
+		cmd.setOtherText( "Disconnect others, then call" );
+		cmd.setDisposition( QSimCommand::Disconnect );
+                command( cmd, this, SLOT(endSession()) );
+            }
+            break;
+
+            case CallsMenu_Hold:
+            {
+                cmd.setType( QSimCommand::SetupCall );
+                cmd.setDestinationDevice( QSimCommand::Network );
+                cmd.setNumber( "1194" );
+                cmd.setText( "Call 1194?" );
+		cmd.setOtherText( "Hold others, then call" );
+		cmd.setDisposition( QSimCommand::PutOnHold );
+                command( cmd, this, SLOT(endSession()) );
+            }
+            break;
+
+            default:
+                endSession();
+                break;
+        }
+    } else {
+        // Unknown response - just go back to the main menu.
+        endSession();
     }
 }
 
@@ -606,7 +644,7 @@ void DemoSimApplication::sticksGameLoop( const QSimTerminalResponse& resp )
         command( cmd, this, SLOT(startSticksGame()) );
     } else {
         // Probably aborted.
-        mainMenu();
+        endSession();
     }
 }
 
@@ -615,7 +653,7 @@ void DemoSimApplication::sticksGamePlayAgain( const QSimTerminalResponse& resp )
     if ( resp.text() == "Yes" )
         startSticksGame();
     else
-        mainMenu();
+        endSession();
 }
 
 void DemoSimApplication::getInputLoop( const QSimTerminalResponse& resp )
@@ -626,9 +664,9 @@ void DemoSimApplication::getInputLoop( const QSimTerminalResponse& resp )
         cmd.setType( QSimCommand::DisplayText );
         cmd.setDestinationDevice( QSimCommand::Display );
         cmd.setText("Enter code of the company." );
-        command( cmd, this, SLOT(mainMenu()) );
+        command( cmd, this, SLOT(endSession()) );
     } else {
-        mainMenu();
+        endSession();
     }
 }
 
@@ -706,10 +744,10 @@ void DemoSimApplication::toneMenu( const QSimTerminalResponse& resp )
         command( cmd, this, SLOT(sendToneMenu()) );
     } else if ( resp.result() == QSimTerminalResponse::BackwardMove ) {
         // Request to move backward.
-        mainMenu();
+        endSession();
     } else {
         // Unknown response - just go back to the main menu.
-        mainMenu();
+        endSession();
     }
 }
 
@@ -779,7 +817,7 @@ void DemoSimApplication::iconMenu( const QSimTerminalResponse& resp )
     if ( resp.result() == QSimTerminalResponse::Success )
         sendIconMenu();
     else
-        mainMenu();
+        endSession();
 }
 
 void DemoSimApplication::iconSEMenu( const QSimTerminalResponse& resp )
@@ -787,7 +825,7 @@ void DemoSimApplication::iconSEMenu( const QSimTerminalResponse& resp )
     if ( resp.result() == QSimTerminalResponse::Success )
         sendIconSEMenu();
     else
-        mainMenu();
+        endSession();
 }
 
 void DemoSimApplication::displayTextResponse( const QSimTerminalResponse& resp )
@@ -795,15 +833,13 @@ void DemoSimApplication::displayTextResponse( const QSimTerminalResponse& resp )
     QSimCommand cmd;
 
     if ( resp.result() == QSimTerminalResponse::Success ) {
-        if ( immediateResponse )
-            return;
-        mainMenu();
+        endSession();
     } else if ( resp.result() == QSimTerminalResponse::BackwardMove ) {
         // Request to move backward.
-        mainMenu();
+        endSession();
     } else {
         // Unknown response - just go back to the main menu.
-        mainMenu();
+        endSession();
     }
 }
 
@@ -899,13 +935,15 @@ void DemoSimApplication::browserMenu( const QSimTerminalResponse& resp )
             }
             break;
 
-            default:    mainMenu(); break;
+            default:
+                endSession();
+                break;
         }
     } else if ( resp.result() == QSimTerminalResponse::BackwardMove ) {
         // Request to move backward.
-        mainMenu();
+        endSession();
     } else {
         // Unknown response - just go back to the main menu.
-        mainMenu();
+        endSession();
     }
 }
